/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "flash.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>

int main()
{
	unsigned int i;
	uint32_t pa = 0x123456;
	uint16_t data_to_program[4] = { 0x4321, 0x8765, 0x9087, 0x3852 };
	uint16_t programmed_data[4] = { 0, 0, 0, 0 };
	flash_t flash;
	
	intc_init();
	flash_init(&flash);
	uart_lite_init();

	PrintString("Write unlock cycle 1\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x00AA);  /* write unlock cycle 1 */
	PrintString("Write unlock cycle 2\n");
	write_little16((uint16_t *) flash.data + 0x2AA, 0x0055);  /* write unlock cycle 2 */
	PrintString("Write unlock bypass command\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x0020);  /* write unlock bypass command */
	
	/* At this point, programming only takes two write cycles. */
	/* Once you enter Unlock Bypass Mode, do a series of like */
	/* operations (programming or sector erase) and then exit */
	/* Unlock Bypass Mode before beginning a different type of */
	/* operations. */

	for(i = 0; i < 4; i++)
	{
		PrintString("Write program setup command\n");
		write_little16((uint16_t *) flash.data + 0x555, 0x00A0);  /* write program setup command */
		PrintString("Write data #");
		PrintUInt32(i);
		PrintString(" to be programmed (");
		PrintHex16(data_to_program[i]);
		PrintString(")\n");
		write_little16((uint16_t *) flash.data + pa + i, data_to_program[i]);       /* write data to be programmed */
	
		uint16_t written_data;
		do
		{
			written_data = read_little16((uint16_t *) flash.data + pa + i);  /* read back data */
		}
		while(written_data != data_to_program[i]);
	}
	
	for(i = 0; i < 4; i++)
	{
		PrintString("Re-reading and Verifying programmed data: ");
		programmed_data[i] = read_little16((uint16_t *) flash.data + pa + i);  /* read back data */
		PrintString((programmed_data[i] == data_to_program[i]) ? "OK" : "FAILED");
		PrintString(" (");
		PrintHex16(programmed_data[i]);
		PrintString(")\n");
	}
	/* exit unlock bypass */
	PrintString("Exit unlock bypass\n");
	write_little16((uint16_t *) flash.data + 0x000, 0x0090);  
	write_little16((uint16_t *) flash.data + 0x000, 0x0000);
	PrintString("Test is finished\n");
	return 0;
}
