/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "timer.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>

timer_t *timer = 0;
volatile uint32_t capture;

void timer_irq_handler()
{
	if(timer)
	{
		if(read_big32(&timer->tcsr0) & TCSR0_T0INT)
		{
			capture = read_big32(&timer->tlr0) - 1; // read capture
			write_big32(&timer->tcsr0, read_big32(&timer->tcsr0) | TCSR0_T0INT); // acknowlegde interrupt in timer
		}
	}
}

int main()
{
	intc_init();
	uart_lite_init();
	
	timer = timer_init(); // Map in memory and initialize timer
	register_irq_handler(TIMER_IRQ, timer_irq_handler);
	
	write_big32(&timer->tlr0, 1);

	write_big32(&timer->tcsr0, TCSR0_LOAD0); // Move TLR0 into TCR0
	write_big32(&timer->tcsr0, TCSR0_ENT0 | TCSR0_MDT0 | TCSR0_CAPT0 | TCSR0_ENIT0); // enable timer 0, up counter, capture mode, enable interrupt signal, hold capture
	
	enable_irq(TIMER_IRQ);

	uint32_t capture_to_print = 0;
	uint32_t last_printed_capture = 0;
	while(last_printed_capture < 200000)
	{
		capture_to_print = capture;
		if(capture_to_print != last_printed_capture)
		{
			PrintString("Last capture is ");
			PrintUInt32(capture_to_print * TIMER_CYCLE_TIME);
			PrintString(" ns\n");
			last_printed_capture = capture_to_print;
		}
	}
	PrintString("Test is finished\n");
	write_big32(&timer->tcsr0, 0); // disable everything
	disable_irq(TIMER_IRQ);
	return 0;
}
