/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "timer.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>




timer_t *timer = 0;

void sleep()
{
	uint32_t msr;
	__asm__ volatile("mfmsr %0" : "=r" (msr));
	
	msr |= (1 << 18); // MSR[WE]=1
	__asm__ volatile("mtmsr %0" : "=r" (msr));
}

uint32_t get_time_us()
{
	return ((read_big32(&timer->tcr0) - 1) * TIMER_CYCLE_TIME) / 1000;
}

void wait(uint32_t t_us)
{
	uint32_t start = get_time_us();
	uint32_t stop = start + t_us;
	
	while(get_time_us() < stop)
	{
		unsigned int i;
		for(i = 0; i < 10; i++);
	}
}

int main()
{
	intc_init();
	uart_lite_init();
	timer = timer_init();  // Map in memory and initialize timer
	
	write_big32(&timer->tlr0, 1);

	write_big32(&timer->tcsr0, TCSR0_LOAD0); // Move TLR0 into TCR0
	write_big32(&timer->tcsr0, TCSR0_ENT0); // enable timer 0, down counter, generate mode, autoreload timer counter

	unsigned int i;
	for(i = 0; i < 40; i++)
	{
		PrintString("Time is ");
		PrintUInt32(get_time_us());
		PrintString(" us\n");
		wait(100);
	}
	
	write_big32(&timer->tcsr0, 0); // disable everything
	disable_irq(TIMER_IRQ);
	PrintString("Test is finished\n");
	return 0;
}
